!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
#include "LIS_misc.h"
module noah32_dascf_Mod
!BOP
!
! !MODULE: noah32_dascf_Mod
!
! !DESCRIPTION:
!  
! \begin{description}
!  \item[di\_opt]
!  Choice of DI methodology: ``standard''   -- Rodell and Houser 2004, \newline
!                            ``customized'' -- Liu et al 2013
!  \item[di\_addswe]
!  Amount of SWE (mm) to add to model
!  \item[di\_maxmelt]
!  Maximum SWE melt rate (mm/day)
!  \item[di\_minswe]
!  Threshold of model SWE to be removed at once
!  \item[di\_ndaymelt]
!  Length of snowmelt period in days
!  \item[di\_scaobs1]
!  Threshold of observed SCA for snow presence
!  \item[di\_scaobs2]
!  Threshold of observed SCA for snow non-presence
!  \item[di\_swemod]
!  Threshold of model SWE(mm) for snow non-presence
! \end{description}

! !REVISION HISTORY:
!
! !USES:        
! none

  implicit none

  PRIVATE
!-----------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!-----------------------------------------------------------------------------
  public :: noah32_dascf_init
!-----------------------------------------------------------------------------
! !PUBLIC TYPES:
!-----------------------------------------------------------------------------
  public :: noah32_dascf_struc
!EOP
  type, public ::  noah32_dascf_type_dec 
     character*20           :: di_opt
     real                   :: di_addswe
     real                   :: di_maxmelt
     real                   :: di_minswe
     real                   :: di_ndaymelt
     real                   :: di_scaobs1
     real                   :: di_scaobs2
     real                   :: di_swemod
  end type noah32_dascf_type_dec
  
  type(noah32_dascf_type_dec), allocatable :: noah32_dascf_struc(:)
  SAVE
contains
!BOP
! 
! !ROUTINE: noah32_dascf_init
! \label{noah32_dascf_init}
! 
! !INTERFACE:
  subroutine noah32_dascf_init()
! !USES:
    use LIS_coreMod,      only : LIS_rc
    use LIS_logMod,       only : LIS_verify
! !DESCRIPTION:        
!
!EOP
    implicit none
    integer        :: ndas_di  !number of DI instance for SCF DA, yliu17


    allocate(noah32_dascf_struc(LIS_rc%nnest))

    do n=1,LIS_rc%nnest
       
! yliu17 ---------------------------------
! added for assimilating SCF using DI

  if (LIS_rc%ndas .gt. 0) then

     ndas_di = 0
     do i=1,LIS_rc%ndas
         if (LIS_rc%daalg(i) .eq. "Direct insertion") ndas_di=ndas_di+1
     enddo
     if (ndas_di .gt. 1) then
        write(LIS_logunit,*) 'ERR: LIS config file: LIS currently canot multiple DI instances for each run'
        call LIS_endrun
     endif

     if (ndas_di .eq. 1) then
        call ESMF_ConfigFindLabel(LIS_config, "Direct insertion methodology:",rc=rc)
        do n=1,LIS_rc%nnest
           call ESMF_ConfigGetAttribute(LIS_config,noah32_dascf_struc(n)%di_opt,rc=rc)
           call LIS_verify(rc,'Direct insertion methodology: not defined')
        enddo

        call ESMF_ConfigFindLabel(LIS_config, "Amount of SWE (mm) to add to model:",rc=rc)
        do n=1,LIS_rc%nnest
           call ESMF_ConfigGetAttribute(LIS_config,noah32_dascf_struc(n)%di_addswe,rc=rc)
           call LIS_verify(rc,'Amount of SWE (mm) to add to model: not defined')
        enddo

        call ESMF_ConfigFindLabel(LIS_config, "Maximum SWE melt rate (mm/day):",rc=rc)
        do n=1,LIS_rc%nnest
           call ESMF_ConfigGetAttribute(LIS_config,noah32_dascf_struc(n)%di_maxmelt,rc=rc)
           call LIS_verify(rc,'Maximum SWE melt rate (mm/day): not defined')
        enddo

        call ESMF_ConfigFindLabel(LIS_config, "Threshold of model SWE to be removed at once:",rc=rc)
        do n=1,LIS_rc%nnest
           call ESMF_ConfigGetAttribute(LIS_config,noah32_dascf_struc(n)%di_minswe,rc=rc)
           call LIS_verify(rc,'Threshold of model SWE to be removed at once: not defined')
        enddo

        call ESMF_ConfigFindLabel(LIS_config, "Length of snowmelt period in days:",rc=rc)
        do n=1,LIS_rc%nnest
           call ESMF_ConfigGetAttribute(LIS_config,noah32_dascf_struc(n)%di_ndaymelt,rc=rc)
           call LIS_verify(rc,'Length of snowmelt period in days: not defined')
        enddo

        call ESMF_ConfigFindLabel(LIS_config, "Threshold of observed SCF for snow presence:",rc=rc)
        do n=1,LIS_rc%nnest
           call ESMF_ConfigGetAttribute(LIS_config,noah32_dascf_struc(n)%di_scaobs1,rc=rc)
           call LIS_verify(rc,'Threshold of observed SCF for snow presence: not defined')
        enddo

        call ESMF_ConfigFindLabel(LIS_config, "Threshold of observed SCF for snow non-presence:",rc=rc)
        do n=1,LIS_rc%nnest
           call ESMF_ConfigGetAttribute(LIS_config,noah32_dascf_struc(n)%di_scaobs2,rc=rc)
           call LIS_verify(rc,'Threshold of observed SCF for snow non-presence: not defined')
        enddo
        call ESMF_ConfigFindLabel(LIS_config, "Threshold of model SWE(mm) for snow non-presence:",rc=rc)
        do n=1,LIS_rc%nnest
           call ESMF_ConfigGetAttribute(LIS_config,noah32_dascf_struc(n)%di_swemod,rc=rc)
           call LIS_verify(rc,'Threshold of model SWE(mm) for snow non-presence: not defined')
        enddo
     endif
  endif
!yliu17-------------------------------------------------------

  end subroutine noah32_dascf_init

end module noah32_lsmMod
